from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.model.knowledge import KnowledgeSourceEnum, TextSourceConfig
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.KNOWLEDGE_LOADER, KnowledgeSourceEnum.USER_INPUT_TEXT)
class TextLoader(BaseLoader):

    async def load(self) -> str:
        if isinstance(self.knowledge.source_config, TextSourceConfig):
            return self.knowledge.source_config.text
        raise AttributeError(
            "source_config does not have a 'text' attribute for the current type."
        )



from typing import List, Optional, Union

from github import Github
from pydantic import BaseModel


class GitFileElementType(BaseModel):
    path: str
    mode: str
    url: str
    branch: str
    repo_name: str
    size: int
    sha: str


class GithubRepoLoader:
    repo_name: str
    branch_name: Optional[str] = None
    token: Optional[str] = None

    def __init__(
        self,
        repo_name: str,
        branch_name: Union[str, None] = None,
        token: Union[str, None] = None,
    ):
        self.repo_name = repo_name
        self.branch_name = branch_name
        self.token = token
        self.repo = None
        try:
            self._load_repo()
        except Exception as e:
            print(e)
            raise ValueError(
                f"Failed to load repo {self.repo_name} with branch {self.branch_name}."
            )

    def _load_repo(self) -> None:
        g = Github(self.token) if self.token else Github()
        self.repo = g.get_repo(self.repo_name)  # type: ignore
        if not self.branch_name and self.repo:
            self.branch_name = self.repo.default_branch

    def get_file_list(
        self,
    ) -> List[GitFileElementType]:
        file_tree = self.repo.get_git_tree(self.branch_name, recursive=True)  # type: ignore
        file_list = []
        for item in file_tree.tree:
            # type usually be blob, tree, commit. We only care about blob
            if item.type != "blob":
                continue
            if not isinstance(item.size, int):
                print(f"Invalid size for file {item.path}: {item.size}")
                continue
            file_list.append(
                GitFileElementType(
                    sha=item.sha,
                    path=item.path,
                    url=item.url,
                    mode=item.mode,
                    size=item.size,
                    branch=self.branch_name,  # type: ignore[arg-type]
                    repo_name=self.repo_name,
                )
            )
        return file_list



import re
from typing import List

from langchain_text_splitters import CharacterTextSplitter

from whiskerrag_types.interface.splitter_interface import BaseSplitter
from whiskerrag_types.model.knowledge import TextSplitConfig
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.SPLITTER, "text")
class TextSplitter(BaseSplitter[TextSplitConfig, Text]):
    def split(self, content: str, split_config: TextSplitConfig) -> List[Text]:
        config_dict = split_config.model_dump(mode="json")
        separators = config_dict["separators"] or ["\n\n"]
        separator_pattern = "|".join(map(re.escape, separators))
        split_regex = config_dict["split_regex"]
        if split_regex:
            separator_pattern = split_regex
        splitter = CharacterTextSplitter(
            chunk_size=config_dict["chunk_size"],
            chunk_overlap=config_dict["chunk_overlap"],
            separator=separator_pattern,
            is_separator_regex=True,
            keep_separator=config_dict["keep_separator"],
            strip_whitespace=config_dict["strip_whitespace"],
        )
        return splitter.split_text(content)

    def batch_split(
        self, content: List[str], split_config: TextSplitConfig
    ) -> List[List[str]]:
        return [self.split(text, split_config) for text in content]
